/**
 * Copyright (c) 2007-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.microkernel.api.jbi.messaging;

import org.ow2.petals.jbi.messaging.exchange.PetalsMessageExchange;
import org.ow2.petals.microkernel.api.jbi.component.PetalsComponentContext;
import org.ow2.petals.microkernel.api.jbi.messaging.exception.RoutingException;
import org.ow2.petals.microkernel.api.server.FractalHelper;

/**
 * Routes messages from the <code>DeliveryChannel</code> to their destinations.
 * <p>
 * Perform an address resolution, finding the endpoint(s) satisfying the destination information, and select the
 * appropriate transporter to convey the messages. Resolutions are based on defined rules.
 * </p>
 * <p>
 * Routes messages pushed from the <code>Transporter</code> to the target <code>DeliveryChannel</code>.
 * </p>
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author Roland Naudin - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 */
public interface RouterService {

    /**
     * The default router strategy
     */
    public static final String DEFAULT_PLATFORM_STRATEGY = "highest,3,2,1";

    /**
     * The default value for the send attempt number
     */
    public static final short DEFAULT_SEND_ATTEMPTS = 2;

    /**
     * The default value for the delay between to send attempts
     */
    public static final int DEFAULT_SEND_DELAY = 1000;

    /**
     * The default value for the transport queue max size
     */
    public static final int DEFAULT_TRANSPORT_QUEUE_MAX_SIZE = 10000;

    /**
     * The default value for the transport queue offering timeout
     */
    public static final long DEFAULT_TRANSPORT_QUEUE_OFFERING_TIMEOUT = 2500;

    /**
     * Constant used to specify the secure policy. It assures the security of the exchange by encrypting the exchange
     * sent
     */
    public static final String SECURE_POLICY = "secure";

    /**
     * Constant used to specify the reliable policy. It assures the persistence of the exchange. Message are never lost,
     * but there is effort on the duration of the exchange
     */
    public static final String RELIABLE_POLICY = "reliable";

    /**
     * Constant used to specify the fast policy. It assures the fastest exchange processing. If there is a problem
     * during communication, the message can be lost.
     */
    public static final String FAST_POLICY = "fast";

    /**
     * The default QOS policy.
     */
    public static final String DEFAULT_POLICY = FAST_POLICY;

    /**
     * Delay, in milliseconds, to wait before to force to end pending exchanges when stopping the router.
     */
    public static final long DEFAULT_STOP_TRAFFIC_DELAY = 30000;

    /**
     * Delay, in milliseconds, to wait before to force to end pending exchanges when pausing the router.
     */
    public static final long DEFAULT_PAUSE_TRAFFIC_DELAY = 10000;

    /**
     * Name of the server interface of the Fractal component {@value FractalHelper#CONTAINER_CONTROLLER_COMPONENT}
     */
    public static final String FRACTAL_SRV_ITF_NAME = "service";

    /**
     * Logger name of the router component
     */
    public static final String COMPONENT_LOGGER_NAME = "Petals.JBI-Messaging.RouterService";

    /**
     * Constant used to specify the property name to set the router policy. Used only in plaform distribution. The
     * default policy is fast.
     */
    static final String PROPERTY_ROUTER_QOS = "org.ow2.petals.router.qos";

    /**
     * Constant used to specify the property name to set the router send attempt. Used only in plaform distribution. The
     * default attempt is 2.
     */
    static final String PROPERTY_ROUTER_SEND_ATTEMPT = "org.ow2.petals.router.send.attempt";

    /**
     * Constant used to specify the property name to set the delay between router send attempts. Used only in plaform
     * distribution. The default attempt is 1 second.
     */
    static final String PROPERTY_ROUTER_SEND_DELAY = "org.ow2.petals.router.send.delay";

    /**
     * Send the message into the NMR.
     * 
     * The router is responsible of changing the role of the exchange, resolving endpoints and calling other
     * SenderModules and things like that when a message is sent.
     * 
     * @param exchange
     * @throws RoutingException
     *             impossible to send the message
     */
    void sendInNMR(PetalsMessageExchange exchange) throws RoutingException;

    /**
     * Inform that a component is installing. The <code>Router</code> must inform its <code>Transporters</code> about
     * the component installation.
     * 
     * @param componentContext
     *            The component context
     * 
     * @throws RoutingException
     *             impossible to add the component
     */
    void addComponent(PetalsComponentContext componentContext) throws RoutingException;

    /**
     * Inform that a component is uninstalling. The <code>Router</code> must inform its <code>Transporters</code> about
     * the component uninstallation.
     * 
     * @param componentContext
     *            The component context
     * 
     * @throws RoutingException
     *             impossible to remove the component
     */
    void removeComponent(PetalsComponentContext componentContext) throws RoutingException;

    /**
     * <p>
     * Prepare the router to be definitely stopped. It means that:
     * <ul>
     * <li>all ongoing message traffic must terminate,</li>
     * <li>all new message traffic is rejected</li>
     * </ul>
     * </p>
     */
    void stopTraffic();

    /**
     * <p>
     * Wait end of pending exchanges.
     * </p>
     * 
     * @param waitingTime
     *            The max time to wait
     * @return <code>true</code> if all pending exchanges have terminated during the given max wait time.
     *         <code>false</code> when the given max wait time is reached and exchanges are always pending.
     */
    public boolean waitEndOfPendingExchange(final long waitingTime);
}
